const { Pool } = require("pg");
const Joi = require('joi');

const PostgresService = require('./service/service');
const loggerService = require("../../logger/src")("siemens-postgreDB-service");
const pdfConfig = require("./config");

let postgresServiceInstance ;

const configSchema = Joi.object().keys({
  NAME: Joi.string().required(),
  USERNAME: Joi.string().required(),
  PASSWORD: Joi.string().required(),
  HOST: Joi.string().required(),
  PORT: Joi.number().integer().required(),
  idleTimeoutMillis: Joi.number().integer(),
  max: Joi.number().integer(),
  schema: Joi.string()
});


function createService(config) {
  try {
    Joi.assert(config, configSchema, 'Invalid Config Details');
  
    const { NAME, USERNAME, PASSWORD, HOST, PORT, idleTimeoutMillis = pdfConfig.idleTimeoutMillis, max = pdfConfig.max } = config;
    const connectionString = `postgresql://${USERNAME}:${PASSWORD}@${HOST}:${PORT}/${NAME}`;
    const pool = new Pool({ connectionString, idleTimeoutMillis, max });
  
    const postgresServiceInstance = new PostgresService({loggerService, pool});
  
    return postgresServiceInstance;
  } catch (err) {
    loggerService.error(err);
    throw err;
  }
}

function getPostgresService(config) {
  if (postgresServiceInstance) return postgresServiceInstance;
  return postgresServiceInstance = createService(config);
}

module.exports = {
  createService,
  getPostgresService
};